From 4d8bb66e6e5736d4d1d8bfa04400131aaaa15f41 Mon Sep 17 00:00:00 2001
From: Michael Vogt <mvo@debian.org>
Date: Sat, 31 Aug 2013 18:02:12 +0200
Subject: [PATCH] improve CommandLine::GetCommand() to support "apt-get install
 -f -s -- 2vcard" again

---
 apt-pkg/contrib/cmndline.cc | 36 +++++++++++++++---------------------
 1 file changed, 15 insertions(+), 21 deletions(-)

diff --git a/apt-pkg/contrib/cmndline.cc b/apt-pkg/contrib/cmndline.cc
index 2086d91..ffcf09c 100644
--- a/apt-pkg/contrib/cmndline.cc
+++ b/apt-pkg/contrib/cmndline.cc
@@ -42,34 +42,28 @@ CommandLine::~CommandLine()
 char const * CommandLine::GetCommand(Dispatch const * const Map,
       unsigned int const argc, char const * const * const argv)
 {
-   // if there is a -- on the line there must be the word we search for around it
-   // as -- marks the end of the options, just not sure if the command can be
-   // considered an option or not, so accept both
-   for (size_t i = 1; i < argc; ++i)
-   {
-      if (strcmp(argv[i], "--") != 0)
-	 continue;
-      ++i;
-      if (i < argc)
-	 for (size_t j = 0; Map[j].Match != NULL; ++j)
-	    if (strcmp(argv[i], Map[j].Match) == 0)
-	       return Map[j].Match;
-      i -= 2;
-      if (i != 0)
-	 for (size_t j = 0; Map[j].Match != NULL; ++j)
-	    if (strcmp(argv[i], Map[j].Match) == 0)
-	       return Map[j].Match;
-      return NULL;
-   }
-   // no --, so search for the first word matching a command
-   // FIXME: How like is it that an option parameter will be also a valid Match ?
+   bool stop = false;
    for (size_t i = 1; i < argc; ++i)
    {
+      // a "--" marks the end of option processing, but we still check
+      // the next word as it might be a command
+      if (strcmp(argv[i], "--") == 0)
+      {
+         stop = true;
+         continue;
+      }
+      // a option, not a command so we ignore it
       if (*(argv[i]) == '-')
 	 continue;
+
+      // check if its a valid command
       for (size_t j = 0; Map[j].Match != NULL; ++j)
 	 if (strcmp(argv[i], Map[j].Match) == 0)
 	    return Map[j].Match;
+
+      // the previous option was "--"
+      if(stop) 
+         break;
    }
    return NULL;
 }
-- 
1.8.3.2

